/* WSMTPSrv.C - Windows SMTP Server
    - WinSock V1.1 required

   Author: Ian Blenke

Ian Blenke cannot be held responsible for damages, expressed or implied, for
the use of this software. No commercial use can be made of this product
without the consent of the author. No profit of any kind can be made on the
sale or distribution of this program. If you wish to distribute this program
with other samples of WinSock programming, you must first contact the author
so that he can keep accurate records of its usage. If you write any programs
based on this source code, you may not sell them for any profit without the
written consent of the author. If you incorporate this source code into a
public domain program, all the author requires is a notification that "part
of the code was written by Ian Blenke" and some form of notification that
his name was used in the public domain software distribution. This does not
represent a contract on the part of the author. If any issues cannot clearly
be resolved by reading this text, immediately contact the author.

I don't like such agreements, but in today's world of lawyers and lawbreakers
I have little other choice. Enjoy!
*/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "resource.h"

#define WSMTPSRV_C
#include "WSMTPSrv.h"

#undef USE_3D

/*function prototypes*/
void send_baudot(int ch1, int MODE);
void RX();
void TX();

extern int MODE;
extern int COMPORT;
extern int PORT;
unsigned int id_sent = 0;
unsigned int id_telnet = 0;
unsigned int id_500 = 0;
const int TxRTS = 0x02;
const int TxDTR = 0x01;
int TxLine ;
character_count = 1;


 // Nasty Globals
const  char PF1[4] = {0x1B,0x4F,0x50,'\0'};
const  char PF2[4] = {0x1B,0x4F,0x51,'\0'};
HWND MyHWnd;

int opensocket[40] = {0,0,0,0,0,0,0,0,0,0, // record of any open sockets
                      0,0,0,0,0,0,0,0,0,0, // for multiple connects
					  0,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,0,0,0,0,0,0};

/* DEBUGIT();
    Purpose: To print debugging messages to the debugging window/console.
*/
#ifdef USE_DEBUG
VOID DEBUGIT( LPSTR lpString)
{
 static char szErrors[256];

 wsprintf((LPSTR)szErrors, (LPSTR)"WSMTPD: %s\n\r", lpString);
 OutputDebugString( (LPSTR)szErrors );
}
#else
#define DEBUGIT(a)
#endif

/* WinMain();
    Purpose: The entry point for our app.
*/
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
{
 MSG        msg;
 WORD       wVersionRequested;
 WSADATA    wsaData;
 DLGPROC    dlgProc;
 HICON      hIcon=NULL;
 char txline[10];

	lpszCmdLine=lpszCmdLine;
	hInst=hInstance;
 lstrcpy((LPSTR)szAppName, (LPSTR)APP_NAME);

 DEBUGIT("WinMain() Begin");

	COMPORT = 0x2F8;  // Default Com2
	PORT = 2;
	TxLine = TxDTR;
if (sscanf(lpszCmdLine,"%d %s",&PORT,txline) > 0)
{
	switch (PORT)
	{
		case 1 :  COMPORT = 0x3F8; break;
		case 2 :  COMPORT = 0x2F8; break;
		case 3 :  COMPORT = 0x3E8; break;
		case 4 :  COMPORT = 0x3E0; break;
		case 5 :  COMPORT = 0x2F0; break;
		case 6 :  COMPORT = 0x2E8; break;
		case 7 :  COMPORT = 0x2E0; break;
		case 8 :  COMPORT = 0x260; break;
		default:  COMPORT = 0x2F8;
	}
	    if (strncmp(txline,"DTR",3) == 0)
			TxLine = TxDTR;
		if (strncmp(txline,"dtr",3) == 0)
			TxLine = TxDTR;
		if (strncmp(txline,"RTS",3) == 0)
			TxLine = TxRTS;
		if (strncmp(txline,"rts",3) == 0)
			TxLine = TxRTS;
}


 if(hPrevInstance)
 {
  smtpError(IDS_PREV_INSTANCE);
  return((int) NULL);
 }

 wVersionRequested=WSVERSION;
 if(WSAStartup(wVersionRequested, (LPWSADATA)&wsaData))
 {
  smtpError(IDS_NO_WINSOCK);
  return((int) NULL);
 }
 if((LOBYTE(wsaData.wVersion)!=1) ||
    (HIBYTE(wsaData.wVersion)!=1))
 {
  smtpError(IDS_BAD_VERSION);
  WSACleanup();
  DEBUGIT("WinSock version is not v1.1");
  return((int) NULL);
 }

 #ifdef USE_3D
  Ctl3dRegister(hInst);
  Ctl3dAutoSubclass(hInst);
 #endif /*USE_3D*/


 dlgProc=(DLGPROC)MakeProcInstance((FARPROC)DlgProc, hInst);
 hWndDlg=CreateDialog(hInst, (LPSTR)dlgWSMTPSRV,
														GetDesktopWindow(), dlgProc);

  MyHWnd = hWndDlg;

 if(!hWndDlg)
 {
  smtpError(IDS_SMTP_INIT);
#ifdef USE_3D
  Ctl3dUnregister(hInst);
#endif // USE_3D
  WSACleanup();
  DEBUGIT("Modeless dialog window could not be opened");
  return((int) NULL);
 }

 hIcon=LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICONEMPTY));

 if(hIcon != NULL)
 {
  SetProp(hWndDlg, "icon", hIcon);
 }
 else DEBUGIT("IDI_ICONEMPTY Resource not found");

// Handle a Win32 API change
#ifdef GCL_HICON
 SetClassLong(hWndDlg, GCL_HICON, (long) NULL);
#else
 SetClassWord(hWndDlg, GCW_HICON, (WORD) NULL);
#endif /* GCL_HICON */

 ShowWindow(hWndDlg, nCmdShow);

 DEBUGIT("Starting smtpInit");

 if(smtpInit(INI_FILE))
 {
  DestroyWindow(hWndDlg);
//  smtpError(IDS_SMTP_INIT);
#ifdef USE_3D
  Ctl3dUnregister(hInst);
#endif /* USE_3D */
  WSACleanup();
  DEBUGIT("Daemon initialization error");
  return((int) NULL);
 }

 while(GetMessage(&msg, NULL, 0, 0))
 {
  if(!IsDialogMessage(hWndDlg, &msg))
  {
   TranslateMessage(&msg);
   DispatchMessage(&msg);
  }
 }

#ifdef USE_3D
 Ctl3dUnregister(hInst);
#endif

  WSACleanup();

  DEBUGIT("WinMain() End");

  return msg.wParam;
} /* WinMain */


/* LRESULT CALLBACK DlgProc(HWND, UINT, WPARAM);
    Purpose: To handle the main dialog user intervention.
    Given/Returns: Standard Dialog procedure
*/
LRESULT CALLBACK DlgProc(HWND hWnd, UINT Msg, WPARAM wParam,
                      LPARAM lParam)
{
char temp[80];
int i;

 switch(Msg)
 {
   case 501:
   case 500: if (isascii(wParam))
     {
     sprintf(temp,"%c",wParam); // External input
	    for (i=1;i<40;i++)  // connected parties
		    if (opensocket[i])
		  		{
		      send(i,temp,1,0);
			    if (wParam == '\r')
				    send(i,"\n",1,0);
		    }
	 wsprintf((LPSTR)temp, "%d", ++id_500);
     SetDlgItemText(hWnd, ID_500, (LPSTR)temp);
    }
      break;


  case WM_INITDIALOG:
    {
     if(iLogLevel==LOG_LOW)
     {
      CheckRadioButton(hWnd, IDR_NORMAL, IDR_DEBUG,
                       IDR_NORMAL);
     }
     else
     {
      CheckRadioButton(hWnd, IDR_NORMAL, IDR_DEBUG,
                       IDR_DEBUG);
     }
 //    SetDlgItemText(hWnd, IDE_PATH, (LPSTR)szLocalMailPath);
     SetDlgItemText(hWnd, IDE_HOSTNAME, (LPSTR)szLocalHostName);
     wsprintf((LPSTR)temp, "%d", id_sent);
     SetDlgItemText(hWnd, ID_SENT, (LPSTR)temp);
	 wsprintf((LPSTR)temp, "%d", id_telnet);
     SetDlgItemText(hWnd, ID_TELNET, (LPSTR)temp);
	 wsprintf((LPSTR)temp, "%d", id_500);
     SetDlgItemText(hWnd, ID_500, (LPSTR)temp);
     return(TRUE);
    }

  case WM_COMMAND:
  {
   switch(wParam)
   {
    case IDR_NORMAL:
    {
     iLogLevel=LOG_LOW;
     return(TRUE);
    }

    case IDR_DEBUG:
    {
     iLogLevel=LOG_HIGH;
     return(TRUE);
    }

//    case IDB_PATH:
//				{
//      SetDlgItemText(hWndDlg, IDE_PATH, "Button Pushed");
//						return(TRUE);
//    }
   }
			break;

    // All of this, for a changing Icon - Sheesh
			case WM_PAINT:
			{
				if(IsIconic(hWndDlg))
    {
     PAINTSTRUCT ps;
     HDC hDC;
     POINT dwOrg;
     HICON hIcon = GetProp(hWndDlg, "icon");

     if(hIcon != NULL)
     {
						hDC=BeginPaint(hWndDlg, &ps);
      GetWindowOrgEx(hDC,&dwOrg);
      SendMessage(hWndDlg, WM_ICONERASEBKGND, (WPARAM) hDC, 0L);
						DrawIcon(hDC, dwOrg.x+2, dwOrg.y+2, hIcon);
      EndPaint(hWndDlg, &ps);
      return(TRUE);
     }
    }
   }
			break;

   case WM_ERASEBKGND:
   {
    if(IsIconic(hWndDlg)) return(TRUE);
    else return(FALSE);
   }

   case WM_CLOSE:
   {
    RemoveProp(hWndDlg, "icon");
    netClose();
    DestroyWindow(hWndDlg);
    return(TRUE);
   }

   case WM_QUERYDRAGICON:
   {
    return((long)(GetProp(hWndDlg, "icon")));
   }

   /* Periodically check for mail until user reads it */
   case WM_TIMER:
   {
 
	   //// deleted
   }

   /* Don't let the user ReSize/Maximize the window! */
   case WM_INITMENUPOPUP:
   {
    if(HIWORD(lParam))
    {
     HMENU hSystemMenu;

     hSystemMenu = GetSystemMenu(hWndDlg, FALSE);
     EnableMenuItem(hSystemMenu, SC_SIZE, MF_GRAYED);
     EnableMenuItem(hSystemMenu, SC_MAXIMIZE, MF_GRAYED);
    }
   }
   break;

   case WM_DESTROY:
   {
    PostQuitMessage(0);
    return(TRUE);
   }
  }
 }
	return(FALSE);
} /* dlgProc() */


/* void smtpError(int);
    Purpose: To print out a resource string with parameters
    Given:   ResourceID of format string, and variable arguements.
    Returns: Nothing.
*/
void smtpError(int ResourceID, ...)
{
 char szString[MAXLINE];
 char szBuffer[MAXLINE];
 va_list vaArgs;
 LPSTR lpArg1, lpArg2;

 va_start( vaArgs, ResourceID);
 lpArg1 = va_arg( vaArgs, LPSTR);
 lpArg2 = va_arg( vaArgs, LPSTR);

 if(IsIconic(hWndDlg))          // If in server mode, no modals!
 {
  smtpLog(LOG_HIGH | LOG_TIME, ResourceID, vaArgs);
  return;
 }

 if(!LoadString(hInst, ResourceID, (LPSTR)szString,
                MAXLINE))
    return;

 wsprintf((LPSTR)szBuffer, (LPSTR)szString, lpArg1, lpArg2);

 DEBUGIT(szBuffer);
 MessageBeep(MB_ICONEXCLAMATION);
 MessageBox(NULL, (LPSTR)szBuffer, (LPSTR)szAppName,
            MB_OK | MB_ICONEXCLAMATION);
 return;
} /* smtpError */


/* void smtpLog(int, ...);
    Purpose: To print an item in the "log" listbox.
    Given:   Resource ID and variable args.
    Returns: Nothing
*/
void smtpLog(int iLevel, int ResourceID, ...)
{
 char szString[MAXLINE];
 char szLine[MAXLINE];
 char szTime[23];
 LPSTR lpArg1, lpArg2;
 va_list vaArgs;
 DWORD dwCount;

 if((iLevel&(LOG_TIME-1))<iLogLevel) return;          // Only log what we want

 va_start( vaArgs, ResourceID);

 if(!LoadString(hInst, ResourceID, (LPSTR)szLine, MAXLINE))
    return;

 lpArg1=va_arg(vaArgs, LPSTR);
 lpArg2=va_arg(vaArgs, LPSTR);

 wsprintf((LPSTR)szString, (LPSTR)szLine, lpArg1, lpArg2);

 if(iLevel&LOG_TIME)
 {
  netGetTimeAndDate((LPSTR)szTime, sizeof(szTime));
  wsprintf((LPSTR)szLine, "%s > %s", (LPSTR)szTime, (LPSTR)szString);
 }
 else
 {
  lstrcpy((LPSTR)szLine, (LPSTR)szString);
 }
 DEBUGIT(szLine);
 SendDlgItemMessage(hWndDlg, IDL_LOG, LB_INSERTSTRING, 0,
                    (LPARAM)(LPCSTR)szLine);

 dwCount=SendDlgItemMessage(hWndDlg, IDL_LOG, LB_GETCOUNT, 0, 0l);
 if(dwCount>MAXITEMS)
 {
  SendDlgItemMessage(hWndDlg, IDL_LOG, LB_RESETCONTENT, 0, 0l);
  wsprintf((LPSTR)szLine, "%s> Reset Log - over %d lines", (LPSTR)szTime,
           MAXITEMS);
  DEBUGIT(szLine);
  SendDlgItemMessage(hWndDlg, IDL_LOG, LB_INSERTSTRING, 0,
                    (LPARAM)(LPCSTR)szLine);
 }
} /* smtpLog */


/* BOOL smtpInit(PSTR);
    Purpose: To setup the SMTP protocol and structures.
    Given:   A filename.
    Returns: TRUE on error, FALSE if not.
*/
BOOL smtpInit(PSTR pFilename)
{
 ofnMailPath.lStructSize=sizeof(ofnMailPath);
 ofnMailPath.hwndOwner=hWndDlg;
 ofnMailPath.hInstance=hInst;
 szMailPathFilter[0]='\0';
 LoadString(hInst, IDS_FILTER,
            (LPSTR)szMailPathFilter, sizeof(szMailPathFilter));
 ofnMailPath.lpstrFilter=(LPSTR)szMailPathFilter;
 ofnMailPath.nFilterIndex=2;
 ofnMailPath.lpstrCustomFilter=(LPSTR)NULL;
 ofnMailPath.lpstrFile=(LPSTR)szLocalMailPath;
 ofnMailPath.nMaxFile=sizeof(szLocalMailPath);
 ofnMailPath.lpstrFileTitle=(LPSTR)NULL;
 ofnMailPath.lpstrInitialDir=(LPSTR)NULL;
 ofnMailPath.lpstrTitle=(LPSTR)szAppName;
 ofnMailPath.Flags=OFN_CREATEPROMPT | OFN_NOREADONLYRETURN;
 ofnMailPath.lpstrDefExt=(LPSTR)NULL;

    // Initialize network windows & such
 if(netInit()) return(TRUE);

 // Reads INI File
// iLogLevel=GetPrivateProfileInt(SECTION_MAIN, ENTRY_DEBUG,
//                                LOG_HIGH, (LPSTR)pFilename);
// GetPrivateProfileString(SECTION_MAIN, ENTRY_MAILPATH,
//                         (LPSTR)DEFAULTMAILFILE,
//                         (LPSTR)szLocalMailPath,
//                         sizeof(szLocalMailPath),
//                         (LPSTR)pFilename);

 if(iLogLevel==LOG_LOW)
 {
  CheckRadioButton(hWndDlg, IDR_NORMAL, IDR_DEBUG,
                   IDR_NORMAL);
 }
 else
 {
  CheckRadioButton(hWndDlg, IDR_NORMAL, IDR_DEBUG,
                   IDR_DEBUG);
 }
// SetDlgItemText(hWndDlg, IDE_PATH, (LPSTR)szLocalMailPath);
 SetDlgItemText(hWndDlg, IDE_HOSTNAME, (LPSTR)szLocalHostName);

 return(FALSE);
} /* smtpInit */


/* BOOL smtpServer(LPSMTPCLIENT);
    Purpose: To be the heart & soul of the SMTP
             server.
    Given:   pointer to the Client that was active.
    Returns: FALSE
    Notes:   This is the heart & soul of the beast.
*/
BOOL smtpServer(LPSMTPCLIENT lpClient)
{
 static char szLine[MAXSNDBUFF];       // These are still holdovers
 static char szPaddedLine[MAXSNDBUFF]; // They should be replaced
 char Chx[5];
 


 if(!lpClient) return(TRUE);

 DEBUGIT("smtpServer() Begin");

 


    // This is a hack for when a client QUIT's.
 bConnectionQuit=FALSE;

    // Get as many lines as are waiting
 while(!bConnectionQuit && (clientReceiveChar(lpClient,Chx) == 1))
 //      (clientReceiveLine(lpClient, szLine, sizeof(szLine)) >= 0))
      
 {
  
  switch (Chx[0])
  {
  case 0x1B:
   if (strstr(Chx,PF1))
	{
		send(lpClient->sSocket,"\r\n[RX]\r\n",8,0);
		PostMessage(HWND_BROADCAST, 501,(unsigned int) 0xF0, 0L);
		RX();
		clientReceiveChar(lpClient,Chx);
		clientReceiveChar(lpClient,Chx);
		break;
	 }
	if (strstr(Chx,PF2))
	{
	send(lpClient->sSocket,"\r\n[TX]\r\n",8,0);
	PostMessage(HWND_BROADCAST, 501,(unsigned int) 0xF1, 0L);
	TX();
	clientReceiveChar(lpClient,Chx);
	clientReceiveChar(lpClient,Chx);
	break;
	}
  default:
	  if (isascii(Chx[0]))  // tests for an ascii character
	  {
		switch (Chx[0])
		{
		case '\r': // single cr	present on all telnet sessions
//			PostMessage(HWND_BROADCAST, 500,'\r', 0L);
			clientSendString(lpClient, "\r\n", 2);
			send_baudot('\n',MODE);
			character_count = 1;
			break;
		case '\b':
		case '\t':
		case '\n': // eat it, the message 500 handler does it.
			break;
		default:    // all other characters
//			PostMessage(HWND_BROADCAST, 500,Chx[0], 0L);
			clientSendString(lpClient, Chx, 1);
			send_baudot(Chx[0],MODE);
			character_count++;

		} // switch Chx[0]

        if (((character_count > 71) &&  (Chx[0] == ' ')) 
			        || (character_count > 78))
      {
            send_baudot('\n',MODE);
			clientSendString(lpClient, "\r\n", 2);
			character_count = 1;
 
      }



	  }  // isascii

//		for (i=1;i<40;i++)  // connected parties
//		if (opensocket[i] /*&& (i != lpClient->sSocket)*/)
//		{
//			if (Chx[0] == '\r')
//			{
//				send(i,"\r\n",2,0);
//			    send_baudot('\n',MODE);
//			}
//			else
//				send(i,Chx,1,0);   // AA6ED Modification
//		}
  } //end switch
   
  if(bConnectionQuit) break;
 } /* while(lines exist);*/
 DEBUGIT("smtpServer() End");
 return(FALSE);
} /* smtpServer() */





/* LPSMTPCLIENT smtpAddClient(SOCKET);
    Purpose: Allocate a client structure and initilize it.
    Given:   SOCKET of a connection.
    Returns: A negative number on error, or a
             valid SMTPCLIENT index into
             smtpClientsTable
    Reusability notes:
             SMTPCLIENT smtpClientsTable;
             PHASE_INIT,3 MAXBUFF, MAXCLIENTS;
*/
LPSMTPCLIENT smtpAddClient(SOCKET sSocket)
{
 int iRcvSize;
 int iSizeofRcvSize;
 HANDLE hInput, hOutput;
 HCLIENT hClient;
 LPSTR lpInput, lpOutput;
 LPSMTPCLIENT lpClient, lpWalker;

 DEBUGIT("smtpAddClient() Begin");
    // Find the receiver buffer size
 iSizeofRcvSize=sizeof(iRcvSize);
 getsockopt(sSocket, SOL_SOCKET, SO_RCVBUF, (LPSTR)&iRcvSize,
            (LPINT)&iSizeofRcvSize );

    // Make sure we can handle a certain size line at a time
 if(iRcvSize<MAXSNDBUFF) iRcvSize=MAXSNDBUFF;

    // Make sure we can handle up to 2 buffers at a time
 iRcvSize=iRcvSize*2;

    // Allocate the receiver buffer
 hInput=GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
                  iRcvSize);
 if(!hInput) return(NULL);
 lpInput=GlobalLock(hInput);
 if(!lpInput)
 {
  GlobalFree(hInput);
  return(NULL);
 }

    // Allocate the sending buffer
 // The size is permanent, we only need dynamic with incoming
 hOutput=GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
                  MAXSNDBUFF);
 if(!hOutput)
 {
  GlobalUnlock(hInput);
  GlobalFree(hInput);
  return(NULL);
 }
 lpOutput=GlobalLock(hOutput);
 if(!lpOutput)
 {
  GlobalFree(hOutput);
  GlobalUnlock(hInput);
  GlobalFree(hInput);
  return(NULL);
 }

    // Allocate a SMTPCLIENT element
 hClient=GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
                  sizeof(SMTPCLIENT));
 if(!hClient)
 {
  GlobalUnlock(hOutput);
  GlobalFree(hOutput);
  GlobalUnlock(hInput);
  GlobalFree(hInput);
  return(NULL);
 }
 lpClient=(LPSMTPCLIENT)GlobalLock(hClient);
 if(!lpClient)
 {
  GlobalFree(hClient);
  GlobalUnlock(hOutput);
  GlobalFree(hOutput);
  GlobalUnlock(hInput);
  GlobalFree(hInput);
  return(NULL);
 }

 lpClient->sSocket=sSocket;
 lpClient->bHaveFrom=FALSE;
 lpClient->hfFile=HFILE_ERROR;
 lpClient->hClient=hClient;

 lpClient->lpOutputBuffer=lpOutput;
 lpClient->hOutputBuffer=hOutput;
    // iOutputSize is MAXSNDBUFF
 lpClient->lpInputBuffer=lpInput;
 lpClient->hInputBuffer=hInput;
 lpClient->iInputSize=iRcvSize;

 lpClient->fifoOutputStart=0;
 lpClient->fifoOutputStop=0;
 lpClient->fifoInputStart=0;
 lpClient->fifoInputStop=0;
 lpClient->iState=STATE_WAIT_FOR_COMMAND;
 lpClient->iExtendedFlags=ESMTP_NONE;

 lpClient->lpNextClient=NULL;
 lpClient->lpPrevClient=NULL;

    // If this is the main listening socket, initialize List
 if(!smtpClientsHead)
 {
  smtpClientsHead=lpClient;
 }
 else
 {
  lpWalker=smtpClientsHead;
  while(lpWalker->lpNextClient) lpWalker=lpWalker->lpNextClient;
  lpWalker->lpNextClient=lpClient;  // Add new tail segment
  lpClient->lpPrevClient=lpWalker;  // Tell tail who's its parent
 }

 DEBUGIT("smtpAddClient() End");
 return(lpClient);
} /* smtpAddClient */


/* LPSMTPCLIENT smtpSocketToClient(SOCKET sSocket) */
LPSMTPCLIENT smtpSocketToClient(SOCKET sSocket)
{
 LPSMTPCLIENT lpClient;

 if((int)sSocket==SOCKET_ERROR) return(NULL);
 lpClient=smtpClientsHead;

 while(lpClient!=NULL)
 {
  if(lpClient->sSocket==sSocket) return(lpClient);
  lpClient=lpClient->lpNextClient;
 }
 return(lpClient);
} /* smtpSocketToClient() */


/* BOOL smtpDestroyClient(LPSMTPCLIENT);
    Purpose: To release a SMTPCLIENT structure.
    Given:   Linked list element pointer
    Returns: TRUE if error, FALSE if all OK
    Reusability notes:
*/
BOOL smtpDestroyClient(LPSMTPCLIENT lpClient)
{
 //BOOL bLinger = FALSE;
 LPSMTPCLIENT lpNext, lpPrev;

 DEBUGIT("smtpDestroyClient() Begin");
 if(!lpClient) return(TRUE);

    opensocket[lpClient->sSocket] = FALSE;

    // This code was removed as "dead" code. The setsockopt() just sets
    // the SO_LINGER option to what it already should be by default
    // (thanks Bob (rcq@ftp.com))
// setsockopt(lpClient->sSocket,
//            SOL_SOCKET, SO_LINGER, (char FAR *)&bLinger,
//            sizeof(BOOL));
    // This shutdown isn't needed either...
// shutdown(lpClient->sSocket, 2);
 closesocket(lpClient->sSocket);

 smtpLog(LOG_HIGH | LOG_TIME, LOG_DISCONNECT_S,lpClient->sSocket,(LPSTR)lpClient->szPeer);

    // Hack Alert! This is a hack, this is only a hack. If this were
				// real winsock compliant code, it shouldn't have to do this!
    // But it doesn't hurt, and Lanera seems to need it?
    // Since this is non-standard, it is now no longer needed - but I
    // doubt if it will work on Lanera's stack anymore..
 //if(lpClient!=smtpClientsHead)
 //{
 // WSAAsyncSelect(smtpClientsHead->sSocket, hWndMain, NET_ACTIVITY,
 //                FD_ACCEPT | FD_READ | FD_WRITE | FD_CLOSE);
 //}

    // Is the DATA temporary file still open?
 if(lpClient->hfFile!=HFILE_ERROR)
 {
  _lclose(lpClient->hfFile);
 
 }

    // Patch out THIS segment.
 lpNext=lpClient->lpNextClient;
 lpPrev=lpClient->lpPrevClient;
 if(lpPrev) lpPrev->lpNextClient=lpNext;

    // Free all of the memory
 GlobalUnlock(lpClient->hInputBuffer);
 GlobalFree(lpClient->hInputBuffer);
 GlobalUnlock(lpClient->hOutputBuffer);
 GlobalFree(lpClient->hOutputBuffer);
 GlobalUnlock(lpClient->hClient);
 GlobalFree(lpClient->hClient);

 DEBUGIT("smtpDestroyClient() End");
 return(FALSE);
} /* smtpDestroyClient */


/* void smtpSendMessage(LPSMTPCLIENT, int, int, ...);
    Purpose: To send a resource/reply message.
    Given:   index into smtpClientsTable
             SMTP reply number (211, 503)
             Resource ID of the reply string.
             ... arguments
    Returns: Nothing.
    Other:   If the Resource ID String has a '\n' in it,
             this routine will send it line at a time.
*/
void smtpSendMessage(LPSMTPCLIENT lpClient, int SMTPCode, int ResourceID, ...)
{
 char szString[MAXSNDBUFF];
 char szLine[MAXSNDBUFF];
 LPSTR lpArg1, lpArg2, lpArg3;
 PSTR pStart, pStop;
 va_list vaArgs;

 DEBUGIT("smtpSendMessage() Begin");
 if(!lpClient) return;

 va_start( vaArgs, ResourceID);

 if(!LoadString(hInst, ResourceID, (LPSTR)szLine, sizeof(szLine)))
    return;

 lpArg1=va_arg(vaArgs, LPSTR);
 lpArg2=va_arg(vaArgs, LPSTR);
 lpArg3=va_arg(vaArgs, LPSTR);

 wsprintf((LPSTR)szString, (LPSTR)szLine, lpArg1, lpArg2, lpArg3);

 pStart=pStop=szString;
 do     /* Line loop for embedded '\n's */
 {
  while((*pStop!='\0')&&(*pStop!='\n')) pStop++;
  if((*pStop!='\0'))
  {
   *(pStop++)='\0';
   wsprintf((LPSTR)szLine, "%s", (LPSTR)pStart);
   pStart=pStop;
  }
  else
  {
   wsprintf((LPSTR)szLine, "%s", (LPSTR)pStart);
  }
  clientSendLine(lpClient, (LPSTR)szLine, lstrlen((LPSTR)szLine));
  DEBUGIT((LPSTR)szLine);
 //// smtpLog(LOG_HIGH, LOG_SENT_S, (LPCSTR)szLine);  // AA6ED
 } while(*pStop!='\0');
 DEBUGIT("smtpSendMessage() End");
} /* smtpSendMessage */


/* int clientSendLine(LPSMTPCLIENT, LPSTR, int);
    Purpose: Send 1 line to the caller if it exists.
    Given:   SMTPCLIENT index, Line pointer,
             Line size. (for binary "string" sends if needed)
    Returns: 0 if everything isn't ok.
             >0 tells the caller how many characters it sent.
    Reusability requirements:
             Globals: SMTPCLIENT smtpClientsTable[]
             Defines: MAXCLIENTS, MAXBUFF
    Other:   THIS ROUTINE ADDS A CR/LF PAIR!
*/
int clientSendLine(LPSMTPCLIENT lpClient, LPSTR lpLine, int iLine)
{
 LPSTR lpBuffer;
 int fifoStart;
 int fifoStop;
 int fifoWalker;
 int iDest;

    /* Catch illegal arguements */
 if((!lpClient)||(lpLine==NULL)||(iLine==0))
     return(0);

    /* Make things easier to deal with */
 lpBuffer=lpClient->lpOutputBuffer;
 fifoStart=lpClient->fifoOutputStart;
 fifoWalker=fifoStart;
 fifoStop=lpClient->fifoOutputStop;

    /* Make sure there is enough room - Egad */
 if(( (fifoStop<fifoStart) &&
      (((fifoStop+iLine+2)&MAXSNDBUFF)>=fifoStart)
    )            ||
    ( (fifoStop>=fifoStart) &&
      (((fifoStop+iLine+2)/MAXSNDBUFF)==1) &&
      (((fifoStop+iLine+2)%MAXSNDBUFF)>=fifoStart)
    ))
    return(0);

    /* There is room at the end! Copy it in! */
 iDest=0;
 fifoWalker=fifoStop;
 while(iDest<iLine)
 {
  lpBuffer[fifoWalker]=lpLine[iDest];
  fifoWalker=(fifoWalker+1)%MAXSNDBUFF;
  iDest++;
 }
 lpBuffer[fifoWalker]='\r';      // Append CR on output
 fifoWalker=(fifoWalker+1)%MAXSNDBUFF;
 lpBuffer[fifoWalker]='\n';      // Append LF on output
 fifoWalker=(fifoWalker+1)%MAXSNDBUFF;

 lpClient->fifoOutputStop=fifoWalker;
    // This is important
 netSendData(lpClient);
 return(iDest);                  // Tell em how many we copied!
} /* clientSendLine */


int clientSendString(LPSMTPCLIENT lpClient, LPSTR lpLine, int iLine)
{
 LPSTR lpBuffer;
 int fifoStart;
 int fifoStop;
 int fifoWalker;
 int iDest;

    /* Catch illegal arguements */
 if((!lpClient)||(lpLine==NULL)||(iLine==0))
     return(0);

    /* Make things easier to deal with */
 lpBuffer=lpClient->lpOutputBuffer;
 fifoStart=lpClient->fifoOutputStart;
 fifoWalker=fifoStart;
 fifoStop=lpClient->fifoOutputStop;

    /* Make sure there is enough room - Egad */
 if(( (fifoStop<fifoStart) &&
      (((fifoStop+iLine+2)&MAXSNDBUFF)>=fifoStart)
    )            ||
    ( (fifoStop>=fifoStart) &&
      (((fifoStop+iLine+2)/MAXSNDBUFF)==1) &&
      (((fifoStop+iLine+2)%MAXSNDBUFF)>=fifoStart)
    ))
    return(0);

    /* There is room at the end! Copy it in! */
 iDest=0;
 fifoWalker=fifoStop;
 while(iDest<iLine)
 {
  lpBuffer[fifoWalker]=lpLine[iDest];
  fifoWalker=(fifoWalker+1)%MAXSNDBUFF;
  iDest++;
 }
 
 lpClient->fifoOutputStop=fifoWalker;
    // This is important
 netSendData(lpClient);
 return(iDest);                  // Tell em how many we copied!
} /* clientSendString */


/* int clientReceiveLine(LPSMTPCLIENT, LPSTR, int);
    Purpose: Receive 1 line for the caller if it exists.
    Given:   SMTPCLIENT pointer, Line buffer pointer,
             Line buffer size.
    Returns: -1 if no string found in buffer
             0 if just CR or NL
             >0 tells the caller how many characters it was.
    Reusability requirements:
             None!
    Notes:   This routines takes into account backspaces,
             deletes, multiple CRs and LFs.
*/
int clientReceiveLine(LPSMTPCLIENT lpClient, LPSTR lpLine, int iLine)
{
 LPSTR lpBuffer;
 int fifoStart;
 int fifoStop;
 int fifoWalker;
 int iDest;
 int iMax;

 if((!lpClient)||(lpLine==NULL)||(iLine==0))
     return(-1);

     /* Make things easier to deal with */
 lpBuffer=lpClient->lpInputBuffer;
 fifoStart=lpClient->fifoInputStart;
 fifoWalker=fifoStart;
 fifoStop=lpClient->fifoInputStop;
 iMax=lpClient->iInputSize;

 if(fifoStart==fifoStop) return(-1);

    /* Search buffer for a LF or CR BEFORE the end of the buffer! */
 while((lpBuffer[fifoWalker]!='\n')&&
       (lpBuffer[fifoWalker]!='\r'))
 {
  fifoWalker=(fifoWalker+1)%iMax;
  if(fifoWalker==fifoStop)
    return(-1);
 }

    /* We found the end of a string! Copy it out. */
 *lpLine='\0';

 iDest=0;
 fifoWalker=fifoStart;
 while((lpBuffer[fifoWalker]!='\n')&&
       (lpBuffer[fifoWalker]!='\r'))
 {
  if((lpBuffer[fifoWalker]=='\b')||     // Backspace (^H)
     (lpBuffer[fifoWalker]=='\177'))    // Delete    (^?)
  { // Interactive User is talking to us (and making mistakes)
   if((--iDest)<0) iDest=0;             // Backup if we can
   fifoWalker=(fifoWalker+1)%iMax;      // Step over ^H or ^?
   continue;                            // for ^H^H... case
  }
  lpLine[iDest]=lpBuffer[fifoWalker];
  fifoWalker=(fifoWalker+1)%iMax;
  iDest++;
  if(iDest>=iLine)
     return(-1);  // Not enough string space!!!
 }
 lpLine[iDest]='\0';

                               // Step past the '\r'
 if((fifoWalker!=fifoStop)&&
    (lpBuffer[fifoWalker]=='\r')) fifoWalker=(fifoWalker+1)%iMax;

                               // Step past the '\n'
 if((fifoWalker!=fifoStop)&&
    (lpBuffer[fifoWalker]=='\n')) fifoWalker=(fifoWalker+1)%iMax;

 lpClient->fifoInputStart=fifoWalker;
/// smtpLog(LOG_HIGH, LOG_RECEIVED_S, (LPCSTR)lpLine); // AA6ED
 return(iDest);              // Tell em how many we copied!
} /* clientReceiveLine */


int clientReceiveChar(LPSMTPCLIENT lpClient, char * Chx)
{
 LPSTR lpBuffer;
 int fifoStart;
 int fifoStop;
 int fifoWalker;
 int iMax;
 char temp[80];
 

 if(!lpClient)
     return(-1);

     /* Make things easier to deal with */
 lpBuffer=lpClient->lpInputBuffer;
 fifoStart=lpClient->fifoInputStart;
 fifoWalker=fifoStart;
 fifoStop=lpClient->fifoInputStop;
 iMax=lpClient->iInputSize;

 if(fifoStart==fifoStop)  // zero out the buffers
 {
	 lpClient->fifoInputStop = 0;
	 lpClient->fifoInputStart = 0;
	 return(-1);
 }

 if (lstrlen(lpBuffer)>0)
 {
	 	wsprintf((LPSTR)temp, "%d", ++id_telnet);
        SetDlgItemText(MyHWnd, ID_TELNET, (LPSTR)temp);
		lstrcpyn(Chx, &lpBuffer[lpClient->fifoInputStart],4);
		lpClient->fifoInputStart = (lpClient->fifoInputStart+1)%iMax;
			return 1;
 }
 else
	 return 0;
}


/**** End of WSMTPSRV.C ****/
