/* WSMTPSrv.C - Windows SMTP Server V1.70
    - WinSock 1.1 required

   * Handles multiple clients at one time for reliable and quick
     mail service.

   * Uses the Windows Sockets API for wide useability.

   Author: Ian Blenke
   Date:   May 23, 1993

   v1.30a   June 24, 1993    8BITMIME capable.
   v1.40a   June 28, 1993    Unlimited connection capability.
                             Cleaner code. Linked lists.
                             More bugs destroyed.
   v1.70    January 28, 1995 "Ported" source to Turbo C++/Win, fixed the nasty
                             80 character limit problem.
*/

#ifndef WSMTPSRV_H
#define WSMTPSRV_H

#include <Windows.h>
#include <CommDlg.h>
#include <Stdarg.h>
#include <WinSock.h>
#include "Ctl3d.h"
#include "Dialogs.h"
#include "Net.h"

//#define USE_3D          /**/// Uncomment this for 3d Dialog feel
//#define USE_TITLE       /**/// Uncomment this for Dialog title
  			      // bytes sent/received debugging
//#define USE_DEBUG       /**/// Uncommment this for Debugging code

#ifndef APIENTRY        /**/// A Win32ism
#define APIENTRY CALLBACK
#endif /* APIENTRY */

#define APP_NAME        "WSMTPD"
#define CLASS_NETWORK   "SMTPClass"
#define WSVERSION       0x101
#define DEFAULTMAILFILE "c:\\mail.txt"
#define MAXLINE         128
#define MAXFILENAME     144
#define MAXSNDBUFF      2048  // I don't think this app will EVER need
                              // this much of a Sending buffer.
#define MAXITEMS        1000  // Limit the listbox size
#define MAXCLIENTS      5    // Up this number if you need more
#define INI_FILE        "wsmtpd.ini"

#define TIME_CHECK      30000 // 30000ms==30 seconds
#define ID_TIMER        2600  // Misc. timer handle  ;)

    // Miscellanous String Defines
#define STRING_PREFIX   "WSMTP"
#define STRING_UNKNOWN  "Unknown System"
#define STRING_NOMAIL   "WSMTPD v1.70"
#define STRING_HAVEMAIL "WSMTPD v1.70 - Mail!"
#define SECTION_MAIN    "WSMTPD"
#define ENTRY_DEBUG     "Debug"
#define ENTRY_MAILPATH  "Mail"

    // A server connection can be in 2 "states"
#define STATE_WAIT_FOR_COMMAND      0
#define STATE_WAIT_FOR_DOT          1

    // Extended SMTP flag values
#define ESMTP_NONE      0
#define ESMTP_EHLO_USED 1
#define ESMTP_8BITMIME  2
#define ESMTP_SIZE      4

    // Level of reporting
#define LOG_LOW         0
#define LOG_MEDIUM      1
#define LOG_HIGH        2

#define LOG_TIME        8   // Use Time with the log entry

/* Set the module SCOPE of global variables */
#ifdef NET_C
#define SCOPE_NET
#define SCOPE_SRV extern
#else
#define SCOPE_NET extern
#define SCOPE_SRV 
#endif /* NET_C */

extern int opensocket[40];

/* SMTP Command structure - For easy parsing */
typedef struct
{
 PSTR SMTPCommand;
 WORD SMTPCode;             // Tokens for this field are in "Dialogs.h"
} SMTPCODE;

/* Odd hack to keep "phase" debug strings static */
typedef const PSTR PHASESTRING;

/* For Vanities sake                             */
typedef HANDLE HCLIENT;


/* Client structure for multiple connections          */
/* All state flags/variables should be stored in here */
typedef struct tagSMTPCLIENT
{
 SOCKET sSocket;            // Socket handle of the connection
 HCLIENT hClient;           // Handle to "this" client link

 SOCKADDR_IN saPeer;        // Peer name
 char szPeer[MAXLINE];      // Room for the peer name

 HFILE hfFile;              // Handle for the open .TMP file
 char szFile[MAXFILENAME];  // .TMP filename per message
 char szFrom[MAXLINE];      // Room for the Sender's ID
 char szTo[MAXLINE];        // Room for the Recipient's ID
 BOOL bHaveFrom;            // Only let id sender once/message
 int iMessageType;          // MAIL/SEND/SAML/SOML?
 BOOL iState;               // SMTP State (2 states, 0 and 1)
 int iExtendedFlags;        // ESMTP flags

     /* We need buffers to talk with - unique ones at that */
 HANDLE hOutputBuffer;      // Handle to output FIFO buffer
 LPSTR lpOutputBuffer;      // FIFO buffer for output
 int fifoOutputStart;       // Start of buffer where valid
 int fifoOutputStop;        // End of buffer where valid
 int iInputSize;            // Size of FIFO buffer for input
 HANDLE hInputBuffer;       // Handle to input FIFO buffer
 LPSTR lpInputBuffer;       // FIFO buffer for input
 int fifoInputStart;        // Start of buffer where valid
 int fifoInputStop;         // End of buffer where valid

 struct tagSMTPCLIENT FAR *lpPrevClient; // Link to previous client struct
 struct tagSMTPCLIENT FAR *lpNextClient; // Link to next client struct
} SMTPCLIENT;

/* estimated size per structure:
  2+2+4+128+2+144+128+128+2+2+2+2+2+4+2+2+2+2+4+2+2+4+4 == 578 bytes/client
  Not including the separate buffers for each FIFO!

  Know the real bummer? As long as the socket remains open, the memory used
  for the buffers remain locked. Lets just hope noone makes a client that
  doesn't close connection and repetively re-connects. Can you say "Your
  Doctor Watson log is getting quite large"?
*/

typedef SMTPCLIENT FAR *LPSMTPCLIENT;

            /* GLOBALS */
SCOPE_SRV LPSMTPCLIENT smtpClientsHead; // Head of Client's linked list

/* Clients array for future multiple clients at a time support */


#ifdef WSMTPSRV_C
int iLogLevel = LOG_LOW;
#else
extern iLogLevel;
#endif /* WSMTPSRV_C */

	/* The main Task globals */
SCOPE_SRV HINSTANCE hInst;
SCOPE_SRV BOOL bConnectionQuit;
	/* The main Dialog globals */
SCOPE_SRV char szLocalHostName[MAXLINE];
SCOPE_SRV OPENFILENAME ofnMailPath;
SCOPE_SRV char szMailPathFilter[MAXLINE];
SCOPE_SRV char szLocalMailPath[MAXFILENAME];
SCOPE_SRV char szAppName[40];
SCOPE_SRV int iMaxSndBuf;             // Maximum send buffer size
SCOPE_SRV int iMaxRcvBuf;             // Maximum receive buffer size
SCOPE_SRV HWND hWndDlg;               // Main dialog window
SCOPE_SRV HWND hWndMain;              // Main "network" window

    /* PROTOTYPES */

    /* Module Private Prototypes */
#ifdef WSMTPSRV_C
int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
LRESULT CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK IconWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK AboutProc(HWND, UINT, WPARAM, LPARAM);
#endif /* WSMTPSRV_C */

    /* Module Public Prototypes */
SCOPE_SRV void smtpError(int, ...);
SCOPE_SRV void smtpLog(int, int, ...);
SCOPE_SRV BOOL smtpInit(PSTR);
SCOPE_SRV BOOL smtpServer(LPSMTPCLIENT);
SCOPE_SRV LPSMTPCLIENT smtpSocketToClient(SOCKET);
SCOPE_SRV LPSMTPCLIENT smtpAddClient(SOCKET);
SCOPE_SRV BOOL smtpDestroyClient(LPSMTPCLIENT);
SCOPE_SRV void smtpSendMessage(LPSMTPCLIENT, int, int, ...);
SCOPE_SRV int  clientSendLine(LPSMTPCLIENT, LPSTR, int);
SCOPE_SRV int  clientReceiveLine(LPSMTPCLIENT, LPSTR, int);

int clientSendString(LPSMTPCLIENT lpClient, LPSTR lpLine, int iLine);
int clientReceiveChar(LPSMTPCLIENT lpClient, char * Chx);
#endif /* WSMTPSRV_H */
